@@ -14,8 +14,10 @@
 #include "cmCPackLog.h" // IWYU pragma: keep
 #include "cmDuration.h"
 #include "cmGeneratedFileStream.h"
+#include "cmList.h"
 #include "cmStringAlgorithms.h"
 #include "cmSystemTools.h"
+#include "cmValue.h"
 
 cmCPackIFWGenerator::cmCPackIFWGenerator()
 {
@@ -37,202 +39,266 @@ int cmCPackIFWGenerator::PackageFiles()
   std::string ifwTLD = this->GetOption("CPACK_TOPLEVEL_DIRECTORY");
   std::string ifwTmpFile = cmStrCat(ifwTLD, "/IFWOutput.log");
 
-  // Run repogen
-  if (!this->Installer.RemoteRepositories.empty()) {
-    std::vector<std::string> ifwCmd;
-    std::string ifwArg;
-
-    ifwCmd.emplace_back(this->RepoGen);
-
-    if (this->IsVersionLess("2.0.0")) {
-      ifwCmd.emplace_back("-c");
-      ifwCmd.emplace_back(this->toplevel + "/config/config.xml");
-    }
-
-    ifwCmd.emplace_back("-p");
-    ifwCmd.emplace_back(this->toplevel + "/packages");
-
-    if (!this->PkgsDirsVector.empty()) {
-      for (std::string const& it : this->PkgsDirsVector) {
-        ifwCmd.emplace_back("-p");
-        ifwCmd.emplace_back(it);
-      }
-    }
-
-    if (!this->RepoDirsVector.empty()) {
-      if (!this->IsVersionLess("3.1")) {
-        for (std::string const& rd : this->RepoDirsVector) {
-          ifwCmd.emplace_back("--repository");
-          ifwCmd.emplace_back(rd);
-        }
-      } else {
-        cmCPackIFWLogger(WARNING,
-                         "The \"CPACK_IFW_REPOSITORIES_DIRECTORIES\" "
-                           << "variable is set, but content will be skipped, "
-                           << "because this feature available only since "
-                           << "QtIFW 3.1. Please update your QtIFW instance."
-                           << std::endl);
-      }
-    }
-
-    if (!this->OnlineOnly && !this->DownloadedPackages.empty()) {
-      ifwCmd.emplace_back("-i");
-      auto it = this->DownloadedPackages.begin();
-      ifwArg = (*it)->Name;
-      ++it;
-      while (it != this->DownloadedPackages.end()) {
-        ifwArg += "," + (*it)->Name;
-        ++it;
-      }
-      ifwCmd.emplace_back(ifwArg);
-    }
-    ifwCmd.emplace_back(this->toplevel + "/repository");
-    cmCPackIFWLogger(VERBOSE,
-                     "Execute: " << cmSystemTools::PrintSingleCommand(ifwCmd)
-                                 << std::endl);
-    std::string output;
-    int retVal = 1;
-    cmCPackIFWLogger(OUTPUT, "- Generate repository" << std::endl);
-    bool res = cmSystemTools::RunSingleCommand(
-      ifwCmd, &output, &output, &retVal, nullptr, this->GeneratorVerbose,
-      cmDuration::zero());
-    if (!res || retVal) {
-      cmGeneratedFileStream ofs(ifwTmpFile);
-      ofs << "# Run command: " << cmSystemTools::PrintSingleCommand(ifwCmd)
-          << std::endl
-          << "# Output:" << std::endl
-          << output << std::endl;
-      cmCPackIFWLogger(
-        ERROR,
-        "Problem running IFW command: "
-          << cmSystemTools::PrintSingleCommand(ifwCmd) << std::endl
-          << "Please check \"" << ifwTmpFile << "\" for errors" << std::endl);
-      return 0;
-    }
-
-    if (!this->Repository.RepositoryUpdate.empty() &&
-        !this->Repository.PatchUpdatesXml()) {
-      cmCPackIFWLogger(WARNING,
-                       "Problem patch IFW \"Updates\" "
-                         << "file: \"" << this->toplevel
-                         << "/repository/Updates.xml\"" << std::endl);
-    }
-
-    cmCPackIFWLogger(OUTPUT,
-                     "- repository: \"" << this->toplevel
-                                        << "/repository\" generated"
-                                        << std::endl);
+  // Create repositories
+  if (!this->RunRepogen(ifwTmpFile)) {
+    return 0;
   }
 
-  // Run binary creator
-  {
-    std::vector<std::string> ifwCmd;
-    std::string ifwArg;
+  // Create installer
+  if (!this->RunBinaryCreator(ifwTmpFile)) {
+    return 0;
+  }
 
-    ifwCmd.emplace_back(this->BinCreator);
+  return 1;
+}
 
+std::vector<std::string> cmCPackIFWGenerator::BuildRepogenCommand()
+{
+  std::vector<std::string> ifwCmd;
+  std::string ifwArg;
+
+  ifwCmd.emplace_back(this->RepoGen);
+
+  if (!this->IsVersionLess("4.2")) {
+    if (!this->ArchiveFormat.empty()) {
+      ifwCmd.emplace_back("--archive-format");
+      ifwCmd.emplace_back(this->ArchiveFormat);
+    }
+    if (!this->ArchiveCompression.empty()) {
+      ifwCmd.emplace_back("--compression");
+      ifwCmd.emplace_back(this->ArchiveCompression);
+    }
+  }
+
+  if (this->IsVersionLess("2.0.0")) {
     ifwCmd.emplace_back("-c");
     ifwCmd.emplace_back(this->toplevel + "/config/config.xml");
+  }
 
-    if (!this->Installer.Resources.empty()) {
-      ifwCmd.emplace_back("-r");
-      auto it = this->Installer.Resources.begin();
-      std::string path = this->toplevel + "/resources/";
-      ifwArg = path + *it;
-      ++it;
-      while (it != this->Installer.Resources.end()) {
-        ifwArg += "," + path + *it;
-        ++it;
-      }
-      ifwCmd.emplace_back(ifwArg);
+  ifwCmd.emplace_back("-p");
+  ifwCmd.emplace_back(this->toplevel + "/packages");
+
+  if (!this->PkgsDirsVector.empty()) {
+    for (std::string const& it : this->PkgsDirsVector) {
+      ifwCmd.emplace_back("-p");
+      ifwCmd.emplace_back(it);
     }
+  }
 
-    ifwCmd.emplace_back("-p");
-    ifwCmd.emplace_back(this->toplevel + "/packages");
-
-    if (!this->PkgsDirsVector.empty()) {
-      for (std::string const& it : this->PkgsDirsVector) {
-        ifwCmd.emplace_back("-p");
-        ifwCmd.emplace_back(it);
+  if (!this->RepoDirsVector.empty()) {
+    if (!this->IsVersionLess("3.1")) {
+      for (std::string const& rd : this->RepoDirsVector) {
+        ifwCmd.emplace_back("--repository");
+        ifwCmd.emplace_back(rd);
       }
-    }
-
-    if (!this->RepoDirsVector.empty()) {
-      if (!this->IsVersionLess("3.1")) {
-        for (std::string const& rd : this->RepoDirsVector) {
-          ifwCmd.emplace_back("--repository");
-          ifwCmd.emplace_back(rd);
-        }
-      } else {
-        cmCPackIFWLogger(WARNING,
-                         "The \"CPACK_IFW_REPOSITORIES_DIRECTORIES\" "
-                           << "variable is set, but content will be skipped, "
-                           << "because this feature available only since "
-                           << "QtIFW 3.1. Please update your QtIFW instance."
-                           << std::endl);
-      }
-    }
-
-    if (this->OnlineOnly) {
-      ifwCmd.emplace_back("--online-only");
-    } else if (!this->DownloadedPackages.empty() &&
-               !this->Installer.RemoteRepositories.empty()) {
-      ifwCmd.emplace_back("-e");
-      auto it = this->DownloadedPackages.begin();
-      ifwArg = (*it)->Name;
-      ++it;
-      while (it != this->DownloadedPackages.end()) {
-        ifwArg += "," + (*it)->Name;
-        ++it;
-      }
-      ifwCmd.emplace_back(ifwArg);
-    } else if (!this->DependentPackages.empty()) {
-      ifwCmd.emplace_back("-i");
-      ifwArg.clear();
-      // Binary
-      auto bit = this->BinaryPackages.begin();
-      while (bit != this->BinaryPackages.end()) {
-        ifwArg += (*bit)->Name + ",";
-        ++bit;
-      }
-      // Depend
-      auto it = this->DependentPackages.begin();
-      ifwArg += it->second.Name;
-      ++it;
-      while (it != this->DependentPackages.end()) {
-        ifwArg += "," + it->second.Name;
-        ++it;
-      }
-      ifwCmd.emplace_back(ifwArg);
-    }
-    // TODO: set correct name for multipackages
-    if (!this->packageFileNames.empty()) {
-      ifwCmd.emplace_back(this->packageFileNames[0]);
     } else {
-      ifwCmd.emplace_back("installer" + this->OutputExtension);
+      cmCPackIFWLogger(WARNING,
+                       "The \"CPACK_IFW_REPOSITORIES_DIRECTORIES\" "
+                         << "variable is set, but content will be skipped, "
+                         << "because this feature available only since "
+                         << "QtIFW 3.1. Please update your QtIFW instance."
+                         << std::endl);
     }
-    cmCPackIFWLogger(VERBOSE,
-                     "Execute: " << cmSystemTools::PrintSingleCommand(ifwCmd)
-                                 << std::endl);
-    std::string output;
-    int retVal = 1;
-    cmCPackIFWLogger(OUTPUT, "- Generate package" << std::endl);
-    bool res = cmSystemTools::RunSingleCommand(
-      ifwCmd, &output, &output, &retVal, nullptr, this->GeneratorVerbose,
-      cmDuration::zero());
-    if (!res || retVal) {
-      cmGeneratedFileStream ofs(ifwTmpFile);
-      ofs << "# Run command: " << cmSystemTools::PrintSingleCommand(ifwCmd)
-          << std::endl
-          << "# Output:" << std::endl
-          << output << std::endl;
-      cmCPackIFWLogger(
-        ERROR,
-        "Problem running IFW command: "
-          << cmSystemTools::PrintSingleCommand(ifwCmd) << std::endl
-          << "Please check \"" << ifwTmpFile << "\" for errors" << std::endl);
-      return 0;
+  }
+
+  if (!this->OnlineOnly && !this->DownloadedPackages.empty()) {
+    ifwCmd.emplace_back("-i");
+    auto it = this->DownloadedPackages.begin();
+    ifwArg = (*it)->Name;
+    ++it;
+    while (it != this->DownloadedPackages.end()) {
+      ifwArg += "," + (*it)->Name;
+      ++it;
     }
+    ifwCmd.emplace_back(ifwArg);
+  }
+  ifwCmd.emplace_back(this->toplevel + "/repository");
+
+  return ifwCmd;
+}
+
+int cmCPackIFWGenerator::RunRepogen(const std::string& ifwTmpFile)
+{
+  if (this->Installer.RemoteRepositories.empty()) {
+    return 1;
+  }
+
+  std::vector<std::string> ifwCmd = this->BuildRepogenCommand();
+  cmCPackIFWLogger(VERBOSE,
+                   "Execute: " << cmSystemTools::PrintSingleCommand(ifwCmd)
+                               << std::endl);
+  std::string output;
+  int retVal = 1;
+  cmCPackIFWLogger(OUTPUT, "- Generate repository" << std::endl);
+  bool res = cmSystemTools::RunSingleCommand(ifwCmd, &output, &output, &retVal,
+                                             nullptr, this->GeneratorVerbose,
+                                             cmDuration::zero());
+  if (!res || retVal) {
+    cmGeneratedFileStream ofs(ifwTmpFile);
+    ofs << "# Run command: " << cmSystemTools::PrintSingleCommand(ifwCmd)
+        << std::endl
+        << "# Output:" << std::endl
+        << output << std::endl;
+    cmCPackIFWLogger(
+      ERROR,
+      "Problem running IFW command: "
+        << cmSystemTools::PrintSingleCommand(ifwCmd) << std::endl
+        << "Please check \"" << ifwTmpFile << "\" for errors" << std::endl);
+    return 0;
+  }
+
+  if (!this->Repository.RepositoryUpdate.empty() &&
+      !this->Repository.PatchUpdatesXml()) {
+    cmCPackIFWLogger(WARNING,
+                     "Problem patch IFW \"Updates\" "
+                       << "file: \"" << this->toplevel
+                       << "/repository/Updates.xml\"" << std::endl);
+  }
+
+  cmCPackIFWLogger(OUTPUT,
+                   "- repository: \"" << this->toplevel
+                                      << "/repository\" generated"
+                                      << std::endl);
+  return 1;
+}
+
+std::vector<std::string> cmCPackIFWGenerator::BuildBinaryCreatorCommmand()
+{
+  std::vector<std::string> ifwCmd;
+  std::string ifwArg;
+
+  ifwCmd.emplace_back(this->BinCreator);
+
+  if (!this->IsVersionLess("4.2")) {
+    if (!this->ArchiveFormat.empty()) {
+      ifwCmd.emplace_back("--archive-format");
+      ifwCmd.emplace_back(this->ArchiveFormat);
+    }
+    if (!this->ArchiveCompression.empty()) {
+      ifwCmd.emplace_back("--compression");
+      ifwCmd.emplace_back(this->ArchiveCompression);
+    }
+  }
+
+  if (!this->IsVersionLess("3.0")) {
+#ifdef __APPLE__
+    // macOS only
+    std::string signingIdentity = this->Installer.SigningIdentity;
+    if (!signingIdentity.empty()) {
+      ifwCmd.emplace_back("--sign");
+      ifwCmd.emplace_back(signingIdentity);
+    }
+#endif
+  }
+
+  ifwCmd.emplace_back("-c");
+  ifwCmd.emplace_back(this->toplevel + "/config/config.xml");
+
+  if (!this->Installer.Resources.empty()) {
+    ifwCmd.emplace_back("-r");
+    auto it = this->Installer.Resources.begin();
+    std::string path = this->toplevel + "/resources/";
+    ifwArg = path + *it;
+    ++it;
+    while (it != this->Installer.Resources.end()) {
+      ifwArg += "," + path + *it;
+      ++it;
+    }
+    ifwCmd.emplace_back(ifwArg);
+  }
+
+  ifwCmd.emplace_back("-p");
+  ifwCmd.emplace_back(this->toplevel + "/packages");
+
+  if (!this->PkgsDirsVector.empty()) {
+    for (std::string const& it : this->PkgsDirsVector) {
+      ifwCmd.emplace_back("-p");
+      ifwCmd.emplace_back(it);
+    }
+  }
+
+  if (!this->RepoDirsVector.empty()) {
+    if (!this->IsVersionLess("3.1")) {
+      for (std::string const& rd : this->RepoDirsVector) {
+        ifwCmd.emplace_back("--repository");
+        ifwCmd.emplace_back(rd);
+      }
+    } else {
+      cmCPackIFWLogger(WARNING,
+                       "The \"CPACK_IFW_REPOSITORIES_DIRECTORIES\" "
+                         << "variable is set, but content will be skipped, "
+                         << "because this feature available only since "
+                         << "QtIFW 3.1. Please update your QtIFW instance."
+                         << std::endl);
+    }
+  }
+
+  if (this->OnlineOnly) {
+    ifwCmd.emplace_back("--online-only");
+  } else if (!this->DownloadedPackages.empty() &&
+             !this->Installer.RemoteRepositories.empty()) {
+    ifwCmd.emplace_back("-e");
+    auto it = this->DownloadedPackages.begin();
+    ifwArg = (*it)->Name;
+    ++it;
+    while (it != this->DownloadedPackages.end()) {
+      ifwArg += "," + (*it)->Name;
+      ++it;
+    }
+    ifwCmd.emplace_back(ifwArg);
+  } else if (!this->DependentPackages.empty()) {
+    ifwCmd.emplace_back("-i");
+    ifwArg.clear();
+    // Binary
+    auto bit = this->BinaryPackages.begin();
+    while (bit != this->BinaryPackages.end()) {
+      ifwArg += (*bit)->Name + ",";
+      ++bit;
+    }
+    // Depend
+    auto it = this->DependentPackages.begin();
+    ifwArg += it->second.Name;
+    ++it;
+    while (it != this->DependentPackages.end()) {
+      ifwArg += "," + it->second.Name;
+      ++it;
+    }
+    ifwCmd.emplace_back(ifwArg);
+  }
+  // TODO: set correct name for multipackages
+  if (!this->packageFileNames.empty()) {
+    ifwCmd.emplace_back(this->packageFileNames[0]);
+  } else {
+    ifwCmd.emplace_back("installer" + this->OutputExtension);
+  }
+
+  return ifwCmd;
+}
+
+int cmCPackIFWGenerator::RunBinaryCreator(const std::string& ifwTmpFile)
+{
+  std::vector<std::string> ifwCmd = this->BuildBinaryCreatorCommmand();
+  cmCPackIFWLogger(VERBOSE,
+                   "Execute: " << cmSystemTools::PrintSingleCommand(ifwCmd)
+                               << std::endl);
+  std::string output;
+  int retVal = 1;
+  cmCPackIFWLogger(OUTPUT, "- Generate package" << std::endl);
+  bool res = cmSystemTools::RunSingleCommand(ifwCmd, &output, &output, &retVal,
+                                             nullptr, this->GeneratorVerbose,
+                                             cmDuration::zero());
+  if (!res || retVal) {
+    cmGeneratedFileStream ofs(ifwTmpFile);
+    ofs << "# Run command: " << cmSystemTools::PrintSingleCommand(ifwCmd)
+        << std::endl
+        << "# Output:" << std::endl
+        << output << std::endl;
+    cmCPackIFWLogger(
+      ERROR,
+      "Problem running IFW command: "
+        << cmSystemTools::PrintSingleCommand(ifwCmd) << std::endl
+        << "Please check \"" << ifwTmpFile << "\" for errors" << std::endl);
+    return 0;
   }
 
   return 1;
@@ -248,9 +314,9 @@ const char* cmCPackIFWGenerator::GetPackagingInstallPrefix()
     tmpPref += "packages/" + this->GetRootPackageName() + "/data";
   }
 
-  this->SetOption("CPACK_IFW_PACKAGING_INSTALL_PREFIX", tmpPref.c_str());
+  this->SetOption("CPACK_IFW_PACKAGING_INSTALL_PREFIX", tmpPref);
 
-  return this->GetOption("CPACK_IFW_PACKAGING_INSTALL_PREFIX");
+  return this->GetOption("CPACK_IFW_PACKAGING_INSTALL_PREFIX")->c_str();
 }
 
 const char* cmCPackIFWGenerator::GetOutputExtension()
@@ -273,11 +339,11 @@ int cmCPackIFWGenerator::InitializeInternal()
 
   // Look 'binarycreator' executable (needs)
 
-  const char* BinCreatorStr = this->GetOption(BinCreatorOpt);
+  cmValue BinCreatorStr = this->GetOption(BinCreatorOpt);
   if (!BinCreatorStr || cmIsNOTFOUND(BinCreatorStr)) {
     this->BinCreator.clear();
   } else {
-    this->BinCreator = BinCreatorStr;
+    this->BinCreator = *BinCreatorStr;
   }
 
   if (this->BinCreator.empty()) {
@@ -290,16 +356,16 @@ int cmCPackIFWGenerator::InitializeInternal()
 
   // Look 'repogen' executable (optional)
 
-  const char* RepoGenStr = this->GetOption(RepoGenOpt);
-  if (!RepoGenStr || cmIsNOTFOUND(RepoGenStr)) {
+  cmValue repoGen = this->GetOption(RepoGenOpt);
+  if (!repoGen || cmIsNOTFOUND(repoGen)) {
     this->RepoGen.clear();
   } else {
-    this->RepoGen = RepoGenStr;
+    this->RepoGen = *repoGen;
   }
 
   // Framework version
-  if (const char* FrameworkVersionSrt = this->GetOption(FrameworkVersionOpt)) {
-    this->FrameworkVersion = FrameworkVersionSrt;
+  if (cmValue frameworkVersion = this->GetOption(FrameworkVersionOpt)) {
+    this->FrameworkVersion = *frameworkVersion;
   } else {
     this->FrameworkVersion = "1.9.9";
   }
@@ -312,17 +378,24 @@ int cmCPackIFWGenerator::InitializeInternal()
 
   // Additional packages dirs
   this->PkgsDirsVector.clear();
-  if (const char* dirs = this->GetOption("CPACK_IFW_PACKAGES_DIRECTORIES")) {
+  if (cmValue dirs = this->GetOption("CPACK_IFW_PACKAGES_DIRECTORIES")) {
     cmExpandList(dirs, this->PkgsDirsVector);
   }
 
   // Additional repositories dirs
   this->RepoDirsVector.clear();
-  if (const char* dirs =
-        this->GetOption("CPACK_IFW_REPOSITORIES_DIRECTORIES")) {
+  if (cmValue dirs = this->GetOption("CPACK_IFW_REPOSITORIES_DIRECTORIES")) {
     cmExpandList(dirs, this->RepoDirsVector);
   }
 
+  // Archive format and compression level
+  if (cmValue af = this->GetOption("CPACK_IFW_ARCHIVE_FORMAT")) {
+    this->ArchiveFormat = *af;
+  }
+  if (cmValue ac = this->GetOption("CPACK_IFW_ARCHIVE_COMPRESSION")) {
+    this->ArchiveCompression = *ac;
+  }
+
   // Installer
   this->Installer.Generator = this;
   this->Installer.ConfigureFromOptions();
@@ -330,22 +403,22 @@ int cmCPackIFWGenerator::InitializeInternal()
   // Repository
   this->Repository.Generator = this;
   this->Repository.Name = "Unspecified";
-  if (const char* site = this->GetOption("CPACK_DOWNLOAD_SITE")) {
-    this->Repository.Url = site;
+  if (cmValue site = this->GetOption("CPACK_DOWNLOAD_SITE")) {
+    this->Repository.Url = *site;
     this->Installer.RemoteRepositories.push_back(&this->Repository);
   }
 
   // Repositories
-  if (const char* RepoAllStr = this->GetOption("CPACK_IFW_REPOSITORIES_ALL")) {
-    std::vector<std::string> RepoAllVector = cmExpandedList(RepoAllStr);
-    for (std::string const& r : RepoAllVector) {
+  if (cmValue RepoAllStr = this->GetOption("CPACK_IFW_REPOSITORIES_ALL")) {
+    cmList RepoAllList{ RepoAllStr };
+    for (std::string const& r : RepoAllList) {
       this->GetRepository(r);
     }
   }
 
-  if (const char* ifwDownloadAll = this->GetOption("CPACK_IFW_DOWNLOAD_ALL")) {
+  if (cmValue ifwDownloadAll = this->GetOption("CPACK_IFW_DOWNLOAD_ALL")) {
     this->OnlineOnly = cmIsOn(ifwDownloadAll);
-  } else if (const char* cpackDownloadAll =
+  } else if (cmValue cpackDownloadAll =
                this->GetOption("CPACK_DOWNLOAD_ALL")) {
     this->OnlineOnly = cmIsOn(cpackDownloadAll);
   } else {
@@ -374,9 +447,9 @@ int cmCPackIFWGenerator::InitializeInternal()
   }
 
   // Output extension
-  if (const char* optOutExt =
+  if (cmValue optOutExt =
         this->GetOption("CPACK_IFW_PACKAGE_FILE_EXTENSION")) {
-    this->OutputExtension = optOutExt;
+    this->OutputExtension = *optOutExt;
   } else if (sysName == "Darwin") {
     this->OutputExtension = ".dmg";
   } else {
@@ -396,7 +469,7 @@ std::string cmCPackIFWGenerator::GetComponentInstallDirNameSuffix(
   const std::string suffix = "/data";
 
   if (this->componentPackageMethod == this->ONE_PACKAGE) {
-    return std::string(prefix + this->GetRootPackageName() + suffix);
+    return cmStrCat(prefix, this->GetRootPackageName(), suffix);
   }
 
   return prefix +
@@ -508,21 +581,20 @@ std::string cmCPackIFWGenerator::GetRootPackageName()
 {
   // Default value
   std::string name = "root";
-  if (const char* optIFW_PACKAGE_GROUP =
+  if (cmValue optIFW_PACKAGE_GROUP =
         this->GetOption("CPACK_IFW_PACKAGE_GROUP")) {
     // Configure from root group
     cmCPackIFWPackage package;
     package.Generator = this;
-    package.ConfigureFromGroup(optIFW_PACKAGE_GROUP);
+    package.ConfigureFromGroup(*optIFW_PACKAGE_GROUP);
     name = package.Name;
-  } else if (const char* optIFW_PACKAGE_NAME =
+  } else if (cmValue optIFW_PACKAGE_NAME =
                this->GetOption("CPACK_IFW_PACKAGE_NAME")) {
     // Configure from root package name
-    name = optIFW_PACKAGE_NAME;
-  } else if (const char* optPACKAGE_NAME =
-               this->GetOption("CPACK_PACKAGE_NAME")) {
+    name = *optIFW_PACKAGE_NAME;
+  } else if (cmValue optPACKAGE_NAME = this->GetOption("CPACK_PACKAGE_NAME")) {
     // Configure from package name
-    name = optPACKAGE_NAME;
+    name = *optPACKAGE_NAME;
   }
   return name;
 }
@@ -537,17 +609,14 @@ std::string cmCPackIFWGenerator::GetGroupPackageName(
   if (cmCPackIFWPackage* package = this->GetGroupPackage(group)) {
     return package->Name;
   }
-  const char* option =
+  cmValue option =
     this->GetOption("CPACK_IFW_COMPONENT_GROUP_" +
                     cmsys::SystemTools::UpperCase(group->Name) + "_NAME");
-  name = option ? option : group->Name;
+  name = option ? *option : group->Name;
   if (group->ParentGroup) {
     cmCPackIFWPackage* package = this->GetGroupPackage(group->ParentGroup);
     bool dot = !this->ResolveDuplicateNames;
-    if (dot && name.substr(0, package->Name.size()) == package->Name) {
-      dot = false;
-    }
-    if (dot) {
+    if (dot && !cmHasPrefix(name, package->Name)) {
       name = package->Name + "." + name;
     }
   }
@@ -566,8 +635,8 @@ std::string cmCPackIFWGenerator::GetComponentPackageName(
   }
   std::string prefix = "CPACK_IFW_COMPONENT_" +
     cmsys::SystemTools::UpperCase(component->Name) + "_";
-  const char* option = this->GetOption(prefix + "NAME");
-  name = option ? option : component->Name;
+  cmValue option = this->GetOption(prefix + "NAME");
+  name = option ? *option : component->Name;
   if (component->Group) {
     cmCPackIFWPackage* package = this->GetGroupPackage(component->Group);
     if ((this->componentPackageMethod ==
@@ -576,10 +645,7 @@ std::string cmCPackIFWGenerator::GetComponentPackageName(
       return package->Name;
     }
     bool dot = !this->ResolveDuplicateNames;
-    if (dot && name.substr(0, package->Name.size()) == package->Name) {
-      dot = false;
-    }
-    if (dot) {
+    if (dot && !cmHasPrefix(name, package->Name)) {
       name = package->Name + "." + name;
     }
   }
